/**
 * Back 2 Browser Bytecode Translator
 * Copyright (C) 2012 Jaroslav Tulach <jaroslav.tulach@apidesign.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. Look for COPYING file in the top folder.
 * If not, see http://opensource.org/licenses/GPL-2.0.
 */
package org.apidesign.bck2brwsr.launcher;

import org.apidesign.bck2brwsr.launcher.impl.FXBrwsr;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;

import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.application.Platform;
import org.apidesign.bck2brwsr.launcher.impl.JVMBridge;
import org.apidesign.vm4brwsr.Bck2Brwsr;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
final class WebViewLauncher extends Bck2BrwsrLauncher {
    private static final Logger LOG = Logger.getLogger(WebViewLauncher.class.getName());
    static {
        try {
            Method m = URLClassLoader.class.getDeclaredMethod("addURL", URL.class);
            m.setAccessible(true);
            URL l = new URL("file://" + System.getProperty("java.home") + "/lib/jfxrt.jar");
            LOG.log(Level.INFO, "url : {0}", l);
            m.invoke(ClassLoader.getSystemClassLoader(), l);
        } catch (Exception ex) {
            throw new LinkageError("Can't add jfxrt.jar on the classpath", ex);
        }
    }
    
    public WebViewLauncher() {
        super(null);
    }

    @Override
    protected Object[] showBrwsr(final URI url) throws IOException {
        try {
            JVMBridge.registerClassLoaders(loaders());
            Executors.newSingleThreadExecutor().submit(new Runnable() {
                @Override
                public void run() {
                    FXBrwsr.launch(FXBrwsr.class, url.toString());
                }
            });
        } catch (Throwable ex) {
            LOG.log(Level.WARNING, "Can't open WebView", ex);
        }
        return null;
    }
    
    @Override
    void generateBck2BrwsrJS(StringBuilder sb, Bck2Brwsr.Resources loader) throws IOException {
        sb.append("(function() {\n"
            + "  this.jvmBridge = new Array();\n"
            + "  this.jvmBridge.loadClass = Array.prototype.push;\n"
            + "  var bridge = this.jvmBridge;\n"
            + "  this.bck2brwsr = function() { return {\n"
            + "     loadClass : function(name) {\n"
            + "       return bridge.loadClass(name);\n"
            + "     }"
            + "  }; };\n"
            + "})(window);\n"
        );
    }
    
    
    
    @Override
    public void close() throws IOException {
        Platform.exit();
    }
    
}
